/**
 * This {Decorator} class is used to highlight some json and js syntax.
 */
export default class HighlightDecorator {

  /**
   * Default constructor
   * @example Decorators.Highlight.syntax(JSON.stringify(this.data.canvas, null, 2))
   */
  constructor() {
  }
  
  /**
   * This method handles a string and returns an html formated string with colors
   * 
   * credits here: https://stackoverflow.com/questions/4810841/how-can-i-pretty-print-json-using-javascript#answer-7220510
   * @param {string} json - the json string
   * @public
   */
  syntax(json) {
    if (!json) return '';
    json = json.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
    return json.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+-]?\d+)?)/g, (match) => {
      let cls = 'number';
      if (/^"/.test(match)) {
        if (/:$/.test(match)) {
          cls = 'key';
        } else {
          cls = 'string';
        }
      } else if (/true|false/.test(match)) {
        cls = 'boolean';
      } else if (/null/.test(match)) {
        cls = 'null';
      }
      return `<span class="${cls}">${match}</span>`;
    });
  }

}